/*
 * AMFData.cpp
 *
 * Copyright (c) 2007 Dan Weatherford and Facebook, inc.
 * All rights reserved.
 */

#include "AMFData.h"

shared_ptr<AMFData> AMFData::construct(char*& ptr) {
  char typeID = *(ptr++);
  switch ((typeID & 0xff)) {
    case AMF_TYPE_DOUBLE:
      return shared_ptr<AMFData>(new AMFDouble(ptr));
    case AMF_TYPE_BOOLEAN:
      return shared_ptr<AMFData>(new AMFBoolean(ptr));
    case AMF_TYPE_STRING: // AMFString takes a string as a constructor parameter, so we can't use a char* parameter to pass to it for deserialization
      return AMFString::deserialize(ptr); // (as that would implicitly convert to string, which is bad)
    case AMF_TYPE_OBJECT:
      return shared_ptr<AMFData>(new AMFObject(ptr));
    case AMF_TYPE_MIXED_ARRAY:
      return shared_ptr<AMFData>(new AMFMixedArray(ptr));
    case AMF_TYPE_ARRAY:
      return shared_ptr<AMFData>(new AMFArray(ptr));
    case AMF_TYPE_DATE:
      return shared_ptr<AMFData>(new AMFDate(ptr));
  } 
  /// default:
  char errbuf[64];
  sprintf(errbuf, "AMFData::construct: unknown typeID 0x%02x\n", (typeID & 0xff));
  throw std::runtime_error(errbuf);
  return shared_ptr<AMFData>(new AMFData());
}


