/* 
 * fout.h
 *
 * Copyright (c) 2007 Dan Weatherford and Facebook, inc.
 * All rights reserved.
 */

#define BUFFER_SIZE 32768

class fout {
public:
  fout() : fp(NULL), buffer_used(0) {}
  fout(const char* fn) : fp(NULL), buffer_used(0) { this->open(fn); }
  ~fout() { close(); }
  
  bool open(const char* fn) {
    if (fp) this->close();

    fp = fopen(fn, "wb");
    return (fp != NULL);
  }

  operator bool() const {
    return (fp != NULL);
  }

  void flush() {
    if (buffer_used) fwrite(buffer, buffer_used, 1, fp);
    buffer_used = 0;
  }

  void close() {
    if (fp) {
      this->flush();
      fclose(fp);
      fp = NULL;
    }
  }

  void write(const char* dat, size_t len) {
    if ((len + buffer_used) > BUFFER_SIZE) {
      this->flush();
    }
    if (len > BUFFER_SIZE) fwrite(dat, len, 1, fp);
    else {
      memcpy(buffer + buffer_used, dat, len);
      buffer_used += len;
    }
  }

  void putc(char c) {
    if (buffer_used == BUFFER_SIZE) this->flush();
    buffer[buffer_used++] = c;
  }

  uint64_t tell() {
    return (ftello(fp) + ((uint64_t)buffer_used));
  }

  void seek(uint64_t offset, int whence = SEEK_SET) {
    this->flush();
    fseek(fp, offset, whence);
  }

  template <typename T> inline void write(T d) {
    this->write((const char*)&d, sizeof(T));
  }

  template <class Str> inline void write_string(const Str& s) {
    uint32_t sl = s.size();
    this->write((const char*)&sl, sizeof(uint32_t));
    if (sl) this->write(s.data(), sl);
  }

protected:
  FILE* fp;
  uint32_t buffer_used;
  char buffer[BUFFER_SIZE];
private:
  fout(const fout& _r); // noncopyable
  fout& operator=(const fout& _r); // nonassignable
} ;

